#include<GF256.h>

/// PRIVATE DECLARATIONS ///

//unsigned int GF256::inst_counter = 0;

/// PUBLIC DECLARATIONS ///

/** Instantiate a new Element in \f$GF(2^8)\f$
 *
 * \remarks sets the value to 0
 */

 //Erzeugung der log- und Antilog-Tabellen in head.h ausgelagert; so mssen sie nicht in jeder Instanz eines Objekts aus GF256 erzeugt und gespeicehrt werden

extern unsigned char antilog_table[256];
extern unsigned char log_table[256];

GF256::GF256() {
	this->value = 0;
}


/** Initiate a new Element in \f$GF(2^8)\f$
 */
GF256::GF256(unsigned char v) {
	this->value = v % this->modulus;
	if(this->value < 0) this->value += this->modulus;
}


/** Assign an instance of GF256 to this instance
 */
void GF256::operator=(const GF256 &gf) {
	this->value = gf.value;
}


/** Assign an integer value to this instance after converting it to \f$GF(2^8)\f$.
 */
void GF256::operator=(const int v) {
	this->value = v % this->modulus;
}

/** Assigns a unsigned char value to this instance.
 */
void GF256::operator=(unsigned char v) {
	this->value = v;
}


/** Add two elements.
 *
 * <b>Usage Example</b>
 * \code
 * GF256 a = GF256(10);
 * GF256 b = GF256(260);
 *
 * GF256 c = a + b;  // value of c is 14
 * \endcode

 */
GF256 GF256::operator+(const GF256 &gf) {
	return GF256(gf.value ^ this->value);
}


/** Subtraction of two field elements.
 */
GF256 GF256::operator-(const GF256 &gf) {
	return GF256(this->value ^ gf.value);
}


/** XOR of two elements in \f$GF(2^8)\f$
 *
 *\remarks same as addition (GF256::operator+)
 */
GF256 GF256::operator^(const GF256 &gf) {
	return GF256(this->value ^ gf.value);
}


/** Multiply two elements of \f$GF(2^8)\f$.
 *
 * <b>Usage Example:</b>
 * \code
 * GF256 a = GF256(10);
 * GF256 b = GF256(260);
 *
 * GF256 c = a * b;  // value of c is 40
 * \endcode
 */
GF256 GF256::operator*(const GF256 &gf) {

	unsigned char a = this->value;
	unsigned char b = gf.value;
	unsigned char p = 0;
	unsigned char hi_bit_set;

	for(unsigned char counter = 0; counter < 8; counter++) {
		if(b & 1)
			p ^= a;
		hi_bit_set = (a & 0x80);
		a <<= 1;
		if(hi_bit_set)
			a ^= 0x11b;
		b >>= 1;
	}
	return GF256(p);
}


/** Divide two elements of \f$GF(2^8)\f$.
 */
GF256 GF256::operator/(const GF256 &gf) {
	GF256 b = gf;

	return *this * (b.inverse());
}


/** Addition and assignment in one step
 *
 * <b>Usage Example:</b>
 * \code
 * GF256 a = GF256(20);
 * a += GF256(512);
 * \endcode
 */
void GF256::operator+=(const GF256 &gf) {
	this->value = (this->value ^ gf.value);
}


/** Subtraction and assignment
 *
 * \remarks as the field \f$GF(2^8)\f$ has the characteristic 2, this
 *          oparation is equal to addition.
 */
void GF256::operator-=(const GF256 &gf) {
	this->value = (this->value ^ gf.value);
}


/** XOR and assignment
 */
void GF256::operator^=(const GF256 &gf) {
	this->value = (this->value ^ gf.value);
}


/** Multiplication and assignment in one step.
 */
void GF256::operator*=(const GF256 &gf) {
	this->value = (*this * gf).getValue();
}


/** Division and assignment in one step
 */
void GF256::operator/=(const GF256 &gf) {
	this->value = (*this * gf).getValue();
}


/** Return the multiplicative inverse element
 */
GF256 GF256::inverse() {
	if(this->value == 0)
		return GF256(0);
	else
		return GF256(antilog_table[(255 - log_table[this->value])]);
}


/** Get the element value as int
 *
 * \return the element value
 */
unsigned char GF256::getValue() {
	return this->value;
}
